@@ -2,7 +2,7 @@
  * general purpose mouse support for Linux
  *
  * Copyright 1993        ajh@gec-mrc.co.uk (Andrew Haylett)
- * Copyright 1994-1997   rubini@ipvvis.unipv.it (Alessandro Rubini)
+ * Copyright 1994-1997   rubini@linux.it (Alessandro Rubini)
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -71,8 +71,6 @@ int opt_test=DEF_TEST;
 int opt_ptrdrag=DEF_PTRDRAG;
 int opt_kill=0;
 int opt_repeater=0, opt_double=0;
-int opt_kernel;
-int opt_explicittype; /* kmouse defaults to autodetection */
 char *opt_special=NULL; /* special commands, like reboot or such */
 char *consolename = "/dev/tty0";
 static int opt_resize=0; /* not really an option */
@@ -333,7 +331,7 @@ static inline int processMouse(int fd, Gpm_Event *event, Gpm_Type *type,
 {
   char *data;
   static int fine_dx, fine_dy;
-  static int i, m;
+  static int i, j, m;
   static Gpm_Event nEvent;
   static struct vt_stat stat;
   static struct timeval tv1={0,0}, tv2; /* tv1==0: first click is single */
@@ -371,7 +369,10 @@ static inline int processMouse(int fd, Gpm_Event *event, Gpm_Type *type,
     {
     event->dx=event->dy=0;
 
-    FD_ZERO(&fdSet); FD_SET(fd,&fdSet); i=0; do /* cluster loop */
+    nEvent.modifiers = 0; /* some mice set them */
+    FD_ZERO(&fdSet); FD_SET(fd,&fdSet); i=0;
+
+    do /* cluster loop */
       {
       if (   ((data=getMouseData(fd,m_type))==NULL)
 	  || ((*(m_type->fun))(&nEvent,data)==-1) )
@@ -380,6 +381,9 @@ static inline int processMouse(int fd, Gpm_Event *event, Gpm_Type *type,
 	else break;
 	}
 
+      event->modifiers = nEvent.modifiers; /* propagate modifiers */
+
+      /* propagate buttons */
       nEvent.buttons = opt_sequence[nEvent.buttons]&7; /* change the order */
       oldB=newB; newB=nEvent.buttons;
       if (!i) event->buttons=nEvent.buttons;
@@ -390,6 +394,7 @@ static inline int processMouse(int fd, Gpm_Event *event, Gpm_Type *type,
 	break;
 	}
 
+      /* propagate movement */
       if (!(m_type->absolute)) /* mouse */
 	{
 	if (abs(nEvent.dx)+abs(nEvent.dy) > opt_delta)
@@ -461,10 +466,12 @@ static inline int processMouse(int fd, Gpm_Event *event, Gpm_Type *type,
 
   i=open_console(O_RDONLY);
   ioctl(i,VT_GETSTATE,&stat);
+  j = event->modifiers; /* save them */
   event->modifiers=6; /* code for the ioctl */
   if (ioctl(i,TIOCLINUX,&(event->modifiers))<0)
     oops("get_shift_state");
   close(i);
+  event->modifiers |= j; /* add mouse-specific bits */
   event->vc = stat.v_active;
 
   if (oldB==event->buttons)
@@ -777,15 +784,6 @@ int main(int argc, char **argv)
   signal(SIGINT,  gpm_killed);
   signal(SIGUSR1, gpm_killed); /* usr1 is used by a new gpm killing the old */
 
-  if (opt_kernel)
-    while(1)
-      {
-      opt_kill=0;
-      sleep(0xFFFFFF);  /* six months at a time :-) */
-      if (opt_kill)
-	exit(1);
-      }
-
   signal(SIGWINCH,gpm_killed); /* winch can be sent if console is resized */
 
 /*....................................... create your nodes */
@@ -995,11 +993,6 @@ static void gpm_killed(int signo)
     unlink(GPM_NODE_PID);
     unlink(GPM_NODE_CTL);
     }
-  if (opt_kernel)
-    {
-    opt_kill++;
-    return;
-    }
   exit(0);
 }
 
