@@ -1,7 +1,7 @@
 /* -*-mode:C;tab-width:8-*-
  * gpm.h - public include file for gpm-Linux
  *
- * Copyright 1994,1995   rubini@ipvvis.unipv.it (Alessandro Rubini)
+ * Copyright 1994,1995   rubini@linux.it (Alessandro Rubini)
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -21,13 +21,17 @@
 #ifndef _GPM_H_
 #define _GPM_H_
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 /*....................................... Xtermish stuff */
 #define GPM_XTERM_ON \
   printf("%c[?1001s",27), fflush(stdout), /* save old hilit tracking */ \
   printf("%c[?1000h",27), fflush(stdout) /* enable mouse tracking */
 
 #define GPM_XTERM_OFF \
-  printf("%c[?10001",27), fflush(stdout), /* disable mouse tracking */ \
+  printf("%c[?1000l",27), fflush(stdout), /* disable mouse tracking */ \
   printf("%c[?1001r",27), fflush(stdout) /* restore old hilittracking */
 
 /*....................................... Cfg pathnames */
@@ -216,6 +220,11 @@ char *Gpm_GetLibVersion(int *where);
 char *Gpm_GetServerVersion(int *where);
 int   Gpm_GetSnapshot(Gpm_Event *ePtr);
 
+#ifdef __cplusplus
+  };
+#endif
+
+
 #endif /* _GPM_H_ */
 
 
