@@ -1,7 +1,7 @@
 /*
  * gpm-Linux configuration file (server only)
  *
- * Copyright 1994-1996   rubini@ipvvis.unipv.it
+ * Copyright 1994-1996   rubini@linux.it
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -19,8 +19,8 @@
  ********/
 
 #define GPM_NAME    "gpm-Linux"
-#define GPM_RELEASE "1.13"
-#define GPM_DATE    "Sep 1997"
+#define GPM_RELEASE "1.14"
+#define GPM_DATE    "June 1998"
 
 /* Is logging enabled? */
 #undef CONFIG_GPM_LOG
