@@ -2,7 +2,7 @@
  * gpn.c - support functions for gpm-Linux
  *
  * Copyright 1993        ajh@gec-mrc.co.uk (Andrew Haylett)
- * Copyright 1994,1995   rubini@ipvvis.unipv.it (Alessandro Rubini)
+ * Copyright 1994,1995   rubini@linux.it (Alessandro Rubini)
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -49,8 +49,6 @@
 #include "gpmInt.h"
 #include "gpm.h"
 
-#include "kmouse.h"  /* kmouse ioctl */
-
 #define mkfifo(path, mode) (mknod ((path), (mode) | S_IFIFO, 0))
 
 extern int	errno;
@@ -245,7 +243,6 @@ static int usage(char *whofailed)
  "                     a glidepoint mouse, none by default. (mman/ps2 only)\n"
  "    -i interval      max time interval for multiple clicks (default %i)\n"
  "    -k               kill a running gpm, to start X with a busmouse\n"
- "    -K               run in kernel mode, using the kmouse module\n"
  "    -l charset       loads the inword() LUT (default '%s')\n"
  "    -m mouse-device  sets mouse device\n"
  "    -M               enable multiple mouse. Following options refer to\n"
@@ -326,9 +323,8 @@ return 0; /* never */
 /*===================================================================*/
 int cmdline(int argc, char **argv)
 {
-char options[]="a:b:B:d:Dg:hi:Kkl:m:Mo:pqr:Rs:S:t:Tv23";
-int i, opt, fd, kfd;
-struct kmouse_options koptions;
+char options[]="a:b:B:d:Dg:hi:kl:m:Mo:pqr:Rs:S:t:Tv23";
+int i, opt, fd;
 FILE *f;
 static struct {char *in; char *out;} seq[] = {
   {"123","01234567"},
@@ -358,11 +354,11 @@ static struct {char *in; char *out;} seq[] = {
       case 'h': exit(usage(NULL));
       case 'i': opt_time=atoi(optarg); break;
       case 'k': opt_kill++; break;
-      case 'K': opt_kernel++; break;
       case 'l': opt_lut=optarg; break;
       case 'L': opt_lut=optarg; opt_quit++; break;
       case 'm': opt_dev = optarg; break;
-      case 'M': opt_double++; which_mouse=mouse_table+2; break;
+      case 'M': opt_double++; opt_repeater++;
+          which_mouse=mouse_table+2; break;
       case 'o':
           if (!strcmp(optarg,"dtr"))       opt_toggle=TIOCM_DTR;
           else if (!strcmp(optarg,"rts"))  opt_toggle=TIOCM_RTS;
@@ -374,7 +370,7 @@ static struct {char *in; char *out;} seq[] = {
       case 'r':
 	  /* being called responsiveness, I must take the inverse */
 	  opt_scale=atoi(optarg);
-          if (!opt_scale) opt_scale=100; /* a maximum */
+          if (!opt_scale || opt_scale > 100) opt_scale=100; /* the maximum */
 	  else opt_scale=100/opt_scale;
 	  break;
       case 'R': opt_repeater++; break;
@@ -383,7 +379,7 @@ static struct {char *in; char *out;} seq[] = {
           if (optarg) opt_special=optarg;
           else opt_special="";
           break;
-      case 't': opt_type=optarg; opt_explicittype++; break;
+      case 't': opt_type=optarg; break;
       case 'T': opt_test++; break;
       case 'v': printf(GPM_NAME " " GPM_RELEASE ", " GPM_DATE "\n"); exit(0);
       case '2': opt_three=-1; break;
@@ -462,6 +458,8 @@ static struct {char *in; char *out;} seq[] = {
     /* init the device, and change mouse type */
     if (m_type->init)
       m_type=(m_type->init)(fd, m_type->flags, m_type);
+    if (!m_type)
+      oops("mouse initialization");
 
     if (opt_toggle)
       {
@@ -494,13 +492,10 @@ static struct {char *in; char *out;} seq[] = {
 #endif
 
   /* chdir */
-  if (!opt_kernel) /* FIXME -- this is to find kmouse.o */
-    {
-    if (chdir(GPM_NODE_DIR) && mkdir(GPM_NODE_DIR,GPM_NODE_DIR_MODE))
-      oops(GPM_NODE_DIR);
-    if (chdir(GPM_NODE_DIR))
-      oops(GPM_NODE_DIR);      /* well, I must create my directory first */
-    }
+  if (chdir(GPM_NODE_DIR) && mkdir(GPM_NODE_DIR,GPM_NODE_DIR_MODE))
+    oops(GPM_NODE_DIR);
+  if (chdir(GPM_NODE_DIR))
+    oops(GPM_NODE_DIR);      /* well, I must create my directory first */
 
   /* now sign */
   f=fopen(GPM_NODE_PID,"w");
@@ -521,97 +516,7 @@ static struct {char *in; char *out;} seq[] = {
    * symlink(GPM_NODE_CTL,"/tmp/gpmctl");
    */
 
-  if (!opt_kernel)
-    return mouse_table[1].fd; /* the second is handled in the main() */
-
-  /*
-   * Everything from here onwards is setting up for kmouse
-   */
-
-#if 0 /* don't actively call insmod, let kerneld do it */
-  switch(fork()) /* insert the module */
-    {
-    case -1: 
-      fprintf(stderr,"%s: fork(): %s\n",prgname,strerror(errno));
-      exit(1);
-    case 0:
-      execl("/sbin/insmod","insmod","kmouse",NULL);
-      execlp("insmod","insmod","kmouse",NULL);
-      fprintf(stderr,"insmod: %s\n",strerror(errno));
-      exit(1);
-    default:
-      wait(&i);
-    }
-
-  /* 
-   * Unfortunately, insmod doesn't tell why loading fails.
-   * If the module is already there, it's all right. So go on.
-   */
-  if (i)
-    fprintf(stderr,"%s: Warning: insmod failed\n", prgname);
-#endif
-
-  if ( (kfd=open("/dev/gpmctl",O_RDONLY)) < 0 )
-    {
-    fprintf(stderr,"%s: /dev/gpmctl: %s\n", prgname, strerror(errno));
-    exit(1);
-    }
-  if (ioctl(kfd,KMOUSE_GETVER,&i)<0)
-    {
-    fprintf(stderr,"%s: ioctl(/dev/gpmctl): %s\n", prgname, strerror(errno));
-    exit(1);
-    }
-  if (KMOUSE_V_MAJOR(i) != KMOUSE_V_MAJOR(KMOUSE_VERSION_N))
-    {
-    fprintf(stderr,"%s: kmouse version %i.%i not supported: %i.xx needed\n",
-	    prgname,
-	    KMOUSE_V_MAJOR(i),KMOUSE_V_MINOR(i),
-	    KMOUSE_V_MAJOR(KMOUSE_VERSION_N));
-    exit(1);
-    }
-  for (i=1; i <= 1+opt_double; i++)
-    {
-    struct stat buffer;
-    which_mouse=mouse_table+i; /* used to access options */
-    
-    if (stat(opt_dev,&buffer)<0)
-      {
-      fprintf(stderr,"%s: stat(%s): %s\n",prgname,opt_dev,
-	      strerror(errno));
-      exit(1);
-      }
-    koptions.protoid = opt_explicittype ? m_type->protoid : PROTO_UNKNOWN;
-    strncpy(koptions.sequence,opt_sequence,8);
-    koptions.dev=buffer.st_rdev;
-    koptions.three=opt_three;
-    koptions.delta=opt_delta;
-    koptions.accel=opt_accel;
-    koptions.scale=opt_scale;
-    koptions.scaley=opt_scaley; /* FIXME */
-    koptions.time=opt_time;
-    koptions.tap=opt_glidepoint_tap;
-    koptions.ptrdrag=opt_ptrdrag;
-    if (ioctl(kfd,KMOUSE_SETOPT,&koptions)<0)
-      {
-      fprintf(stderr,"%s: ioctl(/dev/gpmctl): %s\n",prgname,strerror(errno));
-      exit(1);
-      }
-    i=MAJOR(buffer.st_dev);     /* compare with another serial port */
-    stat("/dev/ttyS0",&buffer);
-    if (MAJOR(buffer.st_dev)==i)
-      {
-      i=N_MOUSE;
-      if (ioctl(which_mouse->fd,TIOCSETD,&i)<0) 
-	{
-	fprintf(stderr,"%s: Warning: ioctl(TIOCSETD) failed for %s: %s\n",
-		prgname,opt_dev,strerror(errno));
-	}
-      }
-    }
-  close(kfd);
-  chdir("/");
-  return -1; /* no fd to be used */
-  
+  return mouse_table[1].fd; /* the second is handled in the main() */
 }
   
 
