@@ -2,7 +2,7 @@
  *
  * liblow.c - client library - low level (gpm-Linux)
  *
- * Copyright 1994,1995   rubini@ipvvis.unipv.it (Alessandro Rubini)
+ * Copyright 1994,1995   rubini@linux.it (Alessandro Rubini)
  * 
  * xterm management is mostly by jtklehto@stekt.oulu.fi (Janne Kukonlehto)
  *
@@ -97,6 +97,34 @@ static int magic=GPM_MAGIC;
   return 0;
 }
 
+#if (defined(SIGWINCH))
+/* itz Wed Mar 18 11:19:52 PST 1998 hook window change as well */
+
+/* Old SIGWINCH handler. */
+
+static __sighandler_t gpm_saved_winch_hook;
+
+static void gpm_winch_hook (int signum)
+{
+  struct winsize win;
+
+  signal(SIGWINCH, gpm_winch_hook); /* safety measure */
+  if (SIG_IGN != gpm_saved_winch_hook &&
+      SIG_DFL != gpm_saved_winch_hook) {
+    gpm_saved_winch_hook(signum);
+  } /*if*/
+  if (ioctl(gpm_consolefd, TIOCGWINSZ, &win) == -1) {
+    return;
+  } /*if*/
+  if (!win.ws_col || !win.ws_row) {
+    win.ws_col=80; win.ws_row=25;
+  } /*if*/
+  gpm_mx = win.ws_col - gpm_zerobased;
+  gpm_my = win.ws_row - gpm_zerobased;
+}
+
+#endif /* SIGWINCH */
+
 #if (defined(SIGTSTP)) 
 /* itz: support for SIGTSTP */
 
@@ -104,7 +132,7 @@ static int magic=GPM_MAGIC;
 
 static __sighandler_t gpm_saved_suspend_hook;
 
-static void gpm_suspend_hook ()
+static void gpm_suspend_hook (int signum)
 {
 	Gpm_Connect gpm_connect;
 	sigset_t old_sigset;
@@ -268,8 +296,12 @@ Gpm_Stst *new;
       if ((gpm_saved_suspend_hook = signal (SIGTSTP, SIG_IGN)) != SIG_IGN)
 	signal (SIGTSTP, gpm_suspend_hook);
 #endif
+#if (defined(SIGWINCH))
+      /* And the winch hook .. */
+      gpm_saved_winch_hook = signal(SIGWINCH, gpm_winch_hook);
+#endif
+    } /*if*/
     return gpm_fd;
-    }
 
 /*....................................... Error: free all memory */
 err:
@@ -290,7 +322,8 @@ int Gpm_Close(void)
 {
 Gpm_Stst *next;
 
-  if (gpm_fd==-2) /* xterm */
+   gpm_tried=0; /* reset the error flag for next time */
+   if (gpm_fd==-2) /* xterm */
     GPM_XTERM_OFF;
   else            /* linux */
     {
@@ -304,9 +337,14 @@ Gpm_Stst *next;
     if (--gpm_flag) return -1;
     }
 
-  gpm_tried=0;
   if (gpm_fd>=0) close(gpm_fd);
   gpm_fd=-1;
+#ifdef SIGTSTP
+  signal(SIGTSTP, gpm_saved_suspend_hook);
+#endif
+#ifdef SIGWINCH
+  signal(SIGWINCH, gpm_saved_winch_hook);
+#endif
   close(gpm_consolefd);
   gpm_consolefd=-1;
   return 0;
