@@ -48,8 +48,8 @@
 #endif
 
 #define RMEV_NAME "Mev"
-#define RMEV_RELEASE "1.13"
-#define RMEV_DATE "September 1997"
+#define RMEV_RELEASE "1.14"
+#define RMEV_DATE "June 1998"
 
 
 /*===================================================================*/
